@@ -6,25 +6,37 @@
 
 from HTMLParser import HTMLParser
 import re
+import requests
 import urllib
-import urllib2
 
 
 class DirectoryParser(HTMLParser):
     """Class to parse directory listings"""
 
-    def __init__(self, url):
+    def __init__(self, url, authentication=None, timeout=None):
+        self.authentication = authentication
+        self.timeout = timeout
+
+        self.active_url = None
+        self.entries = []
+
         HTMLParser.__init__(self)
 
-        self.entries = [ ]
-        self.active_url = None
+        # Force the server to not send cached content
+        headers = {'Cache-Control': 'max-age=0'}
+        r = requests.get(url, auth=self.authentication,
+                         headers=headers, timeout=self.timeout)
+        r.raise_for_status()
+        self.feed(r.text)
 
-        req = urllib2.urlopen(url, timeout=60)
-        self.feed(req.read())
+    def filter(self, filter):
+        """Filter entries by calling function or applying regex."""
 
-    def filter(self, regex):
-        pattern = re.compile(regex, re.IGNORECASE)
-        return [entry for entry in self.entries if pattern.match(entry)]
+        if hasattr(filter, '__call__'):
+            return [entry for entry in self.entries if filter(entry)]
+        else:
+            pattern = re.compile(filter, re.IGNORECASE)
+            return [entry for entry in self.entries if pattern.match(entry)]
 
     def handle_starttag(self, tag, attrs):
         if not tag == 'a':
