@@ -19,6 +19,7 @@ import mozinfo
 
 from parser import DirectoryParser
 from timezones import PacificTimezone
+from utils import urljoin
 
 import progressbar
 
@@ -80,7 +81,7 @@ class Scraper(object):
     def __init__(self, directory, version, platform=None,
                  application='firefox', locale='en-US', extension=None,
                  authentication=None, retry_attempts=0, retry_delay=10.,
-                 timeout=None):
+                 is_stub_installer=False, timeout=None):
 
         # Private properties for caching
         self._target = None
@@ -94,12 +95,13 @@ class Scraper(object):
         self.authentication = authentication
         self.retry_attempts = retry_attempts
         self.retry_delay = retry_delay
+        self.is_stub_installer = is_stub_installer
         self.timeout_download = timeout
         self.timeout_network = 60.
 
         # build the base URL
         self.application = application
-        self.base_url = '/'.join([BASE_URL, self.application])
+        self.base_url = urljoin(BASE_URL, self.application)
 
         attempt = 0
         while True:
@@ -114,8 +116,14 @@ class Scraper(object):
                     print 'Will retry in %s seconds...' % self.retry_delay
                     time.sleep(self.retry_delay)
                     print "Retrying... (attempt %s)" % attempt
+
                 if attempt >= self.retry_attempts:
-                    raise
+                    if hasattr(e, 'response') and \
+                            e.response.status_code == 404:
+                        message = "Specified build has not been found"
+                        raise NotFoundError(message, e.response.url)
+                    else:
+                        raise
 
     @property
     def binary(self):
@@ -152,8 +160,14 @@ class Scraper(object):
                     print 'Will retry in %s seconds...' % self.retry_delay
                     time.sleep(self.retry_delay)
                     print "Retrying... (attempt %s)" % attempt
+
                 if attempt >= self.retry_attempts:
-                    raise
+                    if hasattr(e, 'response') and \
+                            e.response.status_code == 404:
+                        message = "Specified build has not been found"
+                        raise NotFoundError(message, self.path)
+                    else:
+                        raise
 
         return self._binary
 
@@ -167,13 +181,13 @@ class Scraper(object):
     def final_url(self):
         """Return the final URL of the build"""
 
-        return '/'.join([self.path, self.binary])
+        return urljoin(self.path, self.binary)
 
     @property
     def path(self):
         """Return the path to the build"""
 
-        return '/'.join([self.base_url, self.path_regex])
+        return urljoin(self.base_url, self.path_regex)
 
     @property
     def path_regex(self):
@@ -225,7 +239,7 @@ class Scraper(object):
                 return td.total_seconds()
             else:
                 return (td.microseconds +
-                        (td.seconds + td.days * 24 * 3600) * 10**6) / 10**6
+                        (td.seconds + td.days * 24 * 3600) * 10 ** 6) / 10 ** 6
 
         attempt = 0
 
@@ -364,7 +378,7 @@ class DailyScraper(Scraper):
     def is_build_dir(self, dir):
         """Return whether or not the given dir contains a build."""
 
-        url = '/'.join([self.base_url, self.monthly_build_list_regex, dir])
+        url = urljoin(self.base_url, self.monthly_build_list_regex, dir)
         parser = DirectoryParser(url, authentication=self.authentication,
                                  timeout=self.timeout_network)
 
@@ -378,9 +392,8 @@ class DailyScraper(Scraper):
                 continue
         return False
 
-
     def get_build_info_for_date(self, date, has_time=False, build_index=None):
-        url = '/'.join([self.base_url, self.monthly_build_list_regex])
+        url = urljoin(self.base_url, self.monthly_build_list_regex)
 
         print 'Retrieving list of builds from %s' % url
         parser = DirectoryParser(url, authentication=self.authentication,
@@ -399,8 +412,9 @@ class DailyScraper(Scraper):
             parser.entries = parser.filter(regex)
 
         if not parser.entries:
+            date_format = '%Y-%m-%d-%H-%M-%S' if has_time else '%Y-%m-%d'
             message = 'Folder for builds on %s has not been found' % \
-                self.date.strftime('%Y-%m-%d-%H-%M-%S' if has_time else '%Y-%m-%d')
+                self.date.strftime(date_format)
             raise NotFoundError(message, url)
 
         # If no index has been given, set it to the last build of the day.
@@ -419,14 +433,15 @@ class DailyScraper(Scraper):
                         'linux64': r'\.%(EXT)s$',
                         'mac': r'\.%(EXT)s$',
                         'mac64': r'\.%(EXT)s$',
-                        'win32': r'(\.installer)\.%(EXT)s$',
-                        'win64': r'(\.installer)\.%(EXT)s$'}
+                        'win32': r'(\.installer)%(STUB)s\.%(EXT)s$',
+                        'win64': r'(\.installer)%(STUB)s\.%(EXT)s$'}
         regex = regex_base_name + regex_suffix[self.platform]
 
         return regex % {'APP': self.application,
                         'LOCALE': self.locale,
                         'PLATFORM': self.platform_regex,
-                        'EXT': self.extension}
+                        'EXT': self.extension,
+                        'STUB': '-stub' if self.is_stub_installer else ''}
 
     def build_filename(self, binary):
         """Return the proposed filename with extension for the binary"""
@@ -449,7 +464,7 @@ class DailyScraper(Scraper):
         """Return the regex for the folder containing builds of a month."""
 
         # Regex for possible builds for the given date
-        return r'nightly/%(YEAR)s/%(MONTH)s/' % {
+        return r'nightly/%(YEAR)s/%(MONTH)s' % {
             'YEAR': self.date.year,
             'MONTH': str(self.date.month).zfill(2)}
 
@@ -458,11 +473,11 @@ class DailyScraper(Scraper):
         """Return the regex for the path"""
 
         try:
-            path = ''.join([self.monthly_build_list_regex,
-                            self.builds[self.build_index]])
+            path = urljoin(self.monthly_build_list_regex,
+                           self.builds[self.build_index])
             return path
         except:
-            folder = ''.join([self.base_url, self.monthly_build_list_regex])
+            folder = urljoin(self.base_url, self.monthly_build_list_regex)
             raise NotFoundError("Specified sub folder cannot be found",
                                 folder)
 
@@ -500,10 +515,12 @@ class ReleaseScraper(Scraper):
                  'linux64': r'^%(APP)s-.*\.%(EXT)s$',
                  'mac': r'^%(APP)s.*\.%(EXT)s$',
                  'mac64': r'^%(APP)s.*\.%(EXT)s$',
-                 'win32': r'^%(APP)s.*\.%(EXT)s$',
-                 'win64': r'^%(APP)s.*\.%(EXT)s$'}
-        return regex[self.platform] % {'APP': self.application,
-                                       'EXT': self.extension}
+                 'win32': r'^%(APP)s.*%(STUB)s.*\.%(EXT)s$',
+                 'win64': r'^%(APP)s.*%(STUB)s.*\.%(EXT)s$'}
+        return regex[self.platform] % {
+            'APP': self.application,
+            'EXT': self.extension,
+            'STUB': 'Stub' if self.is_stub_installer else ''}
 
     @property
     def path_regex(self):
@@ -548,7 +565,7 @@ class ReleaseCandidateScraper(ReleaseScraper):
                 self.version)
 
     def get_build_info_for_version(self, version, build_index=None):
-        url = '/'.join([self.base_url, self.candidate_build_list_regex])
+        url = urljoin(self.base_url, self.candidate_build_list_regex)
 
         print 'Retrieving list of candidate builds from %s' % url
         parser = DirectoryParser(url, authentication=self.authentication,
@@ -653,13 +670,11 @@ class TinderboxScraper(Scraper):
 
         if self.date is not None:
             try:
-                self.date = datetime.fromtimestamp(float(self.date),
-                                                   self.timezone)
-                self.timestamp = self.date
-            except:
+                # date is provided in the format 2013-07-23
                 self.date = datetime.strptime(self.date, '%Y-%m-%d')
-        else:
-            self.date = None
+            except:
+                # date is provided as a unix timestamp
+                self.timestamp = self.date
 
         self.locale_build = self.locale != 'en-US'
         # For localized builds we do not have to retrieve the list of builds
@@ -668,18 +683,11 @@ class TinderboxScraper(Scraper):
             self.builds, self.build_index = self.get_build_info_for_index(
                 self.build_index)
 
-            try:
-                self.timestamp = self.builds[self.build_index]
-            except:
-                raise NotFoundError("Specified sub folder cannot be found",
-                                    self.base_url +
-                                    self.monthly_build_list_regex)
-
     @property
     def binary_regex(self):
         """Return the regex for the binary"""
 
-        regex_base_name = r'^%(APP)s-.*\.%(LOCALE)s\.'
+        regex_base_name = r'^%(APP)s-.*\.%(LOCALE)s\.%(PLATFORM)s'
         regex_suffix = {'linux': r'.*\.%(EXT)s$',
                         'linux64': r'.*\.%(EXT)s$',
                         'mac': r'.*\.%(EXT)s$',
@@ -691,6 +699,7 @@ class TinderboxScraper(Scraper):
 
         return regex % {'APP': self.application,
                         'LOCALE': self.locale,
+                        'PLATFORM': PLATFORM_FRAGMENTS[self.platform],
                         'EXT': self.extension}
 
     def build_filename(self, binary):
@@ -728,12 +737,6 @@ class TinderboxScraper(Scraper):
 
         return False
 
-    @property
-    def date_validation_regex(self):
-        """Return the regex for a valid date argument value"""
-
-        return r'^\d{4}-\d{1,2}-\d{1,2}$|^\d+$'
-
     def detect_platform(self):
         """Detect the current platform"""
 
@@ -748,19 +751,21 @@ class TinderboxScraper(Scraper):
         return platform
 
     def get_build_info_for_index(self, build_index=None):
-        url = '/'.join([self.base_url, self.build_list_regex])
+        url = urljoin(self.base_url, self.build_list_regex)
 
         print 'Retrieving list of builds from %s' % url
-
-        # If a timestamp is given, retrieve just that build
-        regex = '^' + self.timestamp + '$' if self.timestamp else r'^\d+$'
-
         parser = DirectoryParser(url, authentication=self.authentication,
                                  timeout=self.timeout_network)
-        parser.entries = parser.filter(regex)
+        parser.entries = parser.filter(r'^\d+$')
 
-        # If date is given, retrieve the subset of builds on that date
-        if self.date is not None:
+        if self.timestamp:
+            # If a timestamp is given, retrieve the folder with the timestamp
+            # as name
+            parser.entries = self.timestamp in parser.entries and \
+                [self.timestamp]
+
+        elif self.date:
+            # If date is given, retrieve the subset of builds on that date
             parser.entries = filter(self.date_matches, parser.entries)
 
         if not parser.entries:
@@ -782,7 +787,7 @@ class TinderboxScraper(Scraper):
         if self.locale_build:
             return self.build_list_regex
 
-        return '/'.join([self.build_list_regex, self.builds[self.build_index]])
+        return urljoin(self.build_list_regex, self.builds[self.build_index])
 
     @property
     def platform_regex(self):
@@ -837,6 +842,11 @@ def cli():
                       choices=PLATFORM_FRAGMENTS.keys(),
                       metavar='PLATFORM',
                       help='Platform of the application')
+    parser.add_option('--stub',
+                      dest='is_stub_installer',
+                      action='store_true',
+                      help='Stub installer. '
+                           'Only applicable to Windows builds.')
     parser.add_option('--type', '-t',
                       dest='type',
                       choices=BUILD_TYPES.keys(),
@@ -946,6 +956,7 @@ def cli():
                         'authentication': (options.username, options.password),
                         'retry_attempts': options.retry_attempts,
                         'retry_delay': options.retry_delay,
+                        'is_stub_installer': options.is_stub_installer,
                         'timeout': options.timeout}
     scraper_options = {
         'candidate': {'build_number': options.build_number,
